<?php

define('XMLRPC_SESSION_EXPIRED_ERROR_CODE', 5);

/**
 * Retrieve XMLRPC session
 *
 * @return string
 */
function _magento_api_get_session() {
	if ($_SESSION['magento_api']['session']) {
		return $_SESSION['magento_api']['session'];
	} else {
		$user = variable_get('magento_api_user', '');
		$key = variable_get('magento_api_key', '');
		// this needed because if user name or key is empty Magento return session expire error
		// instead of access denied error. And when we try to relogin we going to infinite recursion.
		if ($user != '' && $key != '') {
			$_SESSION['magento_api']['session'] = xmlrpc(_magento_api_get_host(), 'login', $user, $key);
			if (variable_get('magento_api_log', 0)) watchdog('magento_api', 'Drupal function = %drupal_function, Magento function = %magento_function, Arguments = %args, Response = %response', array('%drupal_function' => '_magento_api_get_session', '%magento_function' => 'login', '%args' => serialize(0), '%response' => serialize($_SESSION['magento_api']['session'])), WATCHDOG_NOTICE, $link = NULL);
			return (_magento_api_check_no_errors())	? $_SESSION['magento_api']['session'] : false;
		}
		else drupal_set_message(t('Could not get session because the user name or password is empty'), 'error');
	}
}

/**
 * Retrieve XMLRPC host url
 *
 * @return string
 */
function _magento_api_get_host() {
	return variable_get('magento_api_magento_host_url', 'http://magentohost/api/xmlrpc/');
}

/**
 * Check for API errors
 *
 * @return boolean
 */
function _magento_api_check_no_errors() {
	if ($error = xmlrpc_error()) {
		if ($error->code <= 0) {
			$error->message = t('Outgoing HTTP request failed because the socket could not be opened.');
		} else if ($error->code != XMLRPC_SESSION_EXPIRED_ERROR_CODE) {
			// if error not session expired, show message
			// else do nothing, because will try to relogin 
			if (variable_get('magento_api_log', 0)) watchdog('magento_api',$error->code , array($error->message), WATCHDOG_NOTICE, $link = NULL);
			drupal_set_message(t('Could not get state name because the remote site gave an error: %message (@code).', array('%message' => $error->message, '@code' => $error->code)), 'error');
		}
		return false;
	}
	else return true;
}

/**
 * Check for session expire API error
 *
 * @return boolean
 */
function _magento_api_check_session_expire_error() {
	if ($error = xmlrpc_error()) {
		return ($error->code == XMLRPC_SESSION_EXPIRED_ERROR_CODE) ? true : false;
	}
	else return false;
}

/**
 * Clear XMLRPC session
 *
 * @return void
 */
function _magento_api_clear_session() {
	$_SESSION['magento_api']['session'] = NULL;
	xmlrpc_clear_error();
}

/**
 * Synchronize Magento catalog into Drupal taxonomy.
 * 
 * @return void
 */
function magento_api_synchronize_catalog() {
	$tree = magento_api_catalog_category_tree();
	if ($tree) {
		magento_api_synchronize_tree($tree);
	}
}
/**
 * Synchronize tree into Drupal taxonomy.
 * Reflects all changes, like Add, Change, Move or Delete category/term.
 * 
 * @param $tree - Catalog trees
 * 
 * @return void
 */
function magento_api_synchronize_tree($tree) {
	$_SESSION['magento_api']['catalog_taxonomy_relation'] = array();
	$tree = array($tree);
	// recurse walk through Magento catalog
	magento_api_synchronize_walk_catalog_recurse($tree);

	// get taxonomy and walk through to find if any magento category was deleted
	$taxonomy_tree = taxonomy_get_tree(variable_get('magento_api_catalog_vocabulary_id', 0));
	foreach ($taxonomy_tree as $term) {
		$_SESSION['magento_api']['is_term_deleted'] = true;
		$cid = magento_api_get_catalog_id_by_taxonomy_term($term->tid);
		array_walk_recursive($tree, 'magento_api_synchronize_to_delete', $cid);
		
		// if Drupal have terms that Magento have not then delete this term
		if ($_SESSION['magento_api']['is_term_deleted']) {
			taxonomy_del_term($term->tid);
     			db_query('DELETE FROM {magento_catalog} WHERE tid = %d', $term->tid);
		}
	}
}
/**
 * Callback function to walk through to find if any magento category was deleted
 *
 * @param $value - Magento category Id for concrete taxonomy term
 * @param $key - array key of taxonomy term 
 * @param $cid - Magento category Id
 * @return bool
 */
function magento_api_synchronize_to_delete($value, $key, $cid) {
	if ($key == 'category_id') {
		if ($value == $cid) {
			$_SESSION['magento_api']['is_term_deleted'] = false;
			return;
		}
	}
}

/**
 * Function to walk through Magento catalog tree to reflect changes into Drupal taxonomy
 *
 * @param $categories - array of some one level Magento categories 
 * @return void
 */
function magento_api_synchronize_walk_catalog_recurse($categories) {
	foreach ($categories as $category) {
		// Drupal taxonomy term
		$tt = null;
		// Drupal taxonomy term Id
		$tid = magento_api_get_taxonomy_term_by_catalog_id($category['category_id']);
		// Drupal taxonomy term parent Id
		$parent_tid = magento_api_get_taxonomy_term_by_catalog_id($category['parent_id']);
		if ($tid) {
			// try to find term exists 
			$tt = taxonomy_get_term($tid);
		}
		if ($tt) {
			// if term already exists then update it
			$tt = get_object_vars($tt);
			$tt['name'] = $category['name'];
			$tt['parent'] = $parent_tid;
			$st = taxonomy_save_term($tt);
		} else {
			// if term not exists then creates new
			$tt = array(
				'name' => $category['name'],
				'parent' => $parent_tid,
				'vid' => variable_get('magento_api_catalog_vocabulary_id', 0),
				'weight' => intval($category['position']),
			);
			$st = taxonomy_save_term($tt);
			if ($st == SAVED_NEW) {
				// add new Drupal taxonomy relation to Magento catalog 
				$mc = array(
					'tid' => $tt['tid'],
					'cid' => intval($category['category_id']),
				);
				drupal_write_record('magento_catalog', $mc);
			}
		}
		if (!empty($category['children'])) {
			// if category have children then repeat all actions for they (recurse)
			magento_api_synchronize_walk_catalog_recurse($category['children']);
		}
	}
}

/**
 * Get Drupal taxonomy term by Magento category Id
 *
 * @param $cid - category Id 
 * @return int
 */
function magento_api_get_taxonomy_term_by_catalog_id($cid) {
	return db_result(db_query('SELECT mc.tid
								FROM {magento_catalog} mc
								WHERE mc.cid = %d ', $cid));
}

/**
 * Get Magento category Id by Drupal taxonomy term 
 *
 * @param $tid - term Id 
 * @return int
 */
function magento_api_get_catalog_id_by_taxonomy_term($tid) {
	return db_result(db_query('SELECT mc.cid
								FROM {magento_catalog} mc
								WHERE mc.tid = %d ', $tid));
}
